local L = LibStub("AceLocale-3.0"):GetLocale("JeypLoathebHealSorter")
local AceGUI = LibStub("AceGUI-3.0")
local LSM = LibStub( "LibSharedMedia-3.0" )
JeypLoathebHealSorter.Dialog = LibStub("AceConfigDialog-3.0")

local defaults = {
	global = {
		tFramePosition = {
			fX = 0,
			fY = 0,
		},
		bIgnoreNonParty = false,
		bTestMode = false,
		bSendData = false,
		iRaidWarningCallouts = 1,
		sHealerList = "Jheyp - Jeyp - Kazolk - Boulette - Zengu - Deshiken",
		iDataTimestamp = 0,
		iHealersToDisplay = 5,
		sSeparator = ",",
		iFontSize = 20,
		iMargin = 10,
		sFont = "Arial Narrow",
		sBackground = "None",
		tBackgroundColor = {
			r = 1.0,
			g = 1.0,
			b = 1.0,
			a = 0.0,
		},
		sBorder = "None",
		tBorderColor = {
			r = 1.0,
			g = 1.0,
			b = 1.0,
			a = 0.0,
		},
		tLog = {
		},
	},
}

JeypLoathebHealSorter.options = {
	name = "JeypLoathebHealSorter - Options",
	handler = JeypLoathebHealSorter,
	type = 'group',
	args = {
		testMode = {
			type = "toggle",
			name = L["Test Mode"],
			order = 1,
			get = "getTestMode",
			set = "setTestMode",
		},
		ignoreNonParty = {
			type = "toggle",
			name = L["Ignore Healers not in raid"],
			order = 2,
			get = "getIgnoreNonParty",
			set = "setIgnoreNonParty",
		},
		healersToDisplay = {
			type = "range",
			name = L["Healers To Display"],
			order = 3,
			get = "getHealersToDisplay",
			set = "setHealersToDisplay",
			min = 1,
			max = 15,
			softMin = 1,
			softMax = 15,
			step = 1,
			width = "full",
		},
		WindowAppearance = {
		  name = L["Display"],
		  type = "group",
		  inline = true,
		  order = 4,
		  args={
			bg = {
				order = 1,
				name = L["Background"],
				type = "select",
				dialogControl = "LSM30_Background",
				values = LSM:HashTable( LSM.MediaType.BACKGROUND ),
				get = "getBackground",
				set = "setBackground",
				width = 1.,
			},
			bgColor = {
				order = 2,
				name = L["Background Color"],
				type = "color",
				hasAlpha = true,
				get = "getBackgroundColor",
				set = "setBackgroundColor",
				width = 1.,
			},
			border = {
				order = 3,
				name = L["Border"],
				type = "select",
				dialogControl = "LSM30_Border",
				values = LSM:HashTable( LSM.MediaType.BORDER ),
				get = "getBorder",
				set = "setBorder",
				width = 1.,
			},
			borderColor = {
				order = 4,
				name = L["Border Color"],
				type = "color",
				hasAlpha = true,
				get = "getBorderColor",
				set = "setBorderColor",
				width = 1.,
			},
			margin = {
				order = 5,
				type = "range",
				min = 1,
				max = 15,
				softMin = 1,
				softMax = 15,
				step = 1,
				name = L["Margin"],
				get = "getMargin",
				set = "setMargin",
				width = 1.,
			},
			fontSize = {
				type = "range",
				name = L["Font Size"],
				order = 6,
				get = "getFontSize",
				set = "setFontSize",
				min = 1,
				max = 40,
				softMin = 1,
				softMax = 40,
				step = 0.1,
				width = 1.,
			},
			font = {
				order = 7,
				name = L["Font"],
				type = "select",
				dialogControl = "LSM30_Font",
				values = LSM:HashTable( LSM.MediaType.FONT ),
				get = "getFont",
				set = "setFont",
				width = "full",
			},
		  },
		},
		spacern = {
			type = "description",
			order = 5,
			name = " ",
			width = "full",
		},
		HealerList = {
		  name = L["Data Spreading - RL/Assistant only"],
		  type = "group",
		  inline = true,
		  order = 6,
		  args={
			sendData = {
				type = "toggle",
				name = L["Send Data To Raid"],
				order = 1,
				get = "getSendData",
				set = "setSendData",
			},
			raidWarningCallouts = {
				type = "select",
				name = L["Call out next healer"],
				order = 2,
				values = {
					[0] = L["Disabled"],
					[1] = L["Raid Warning"],
					[2] = L["Whispers"],
				},
				get = "getRaidWarningCallouts",
				set = "setRaidWarningCallouts",
			},
			separator = {
				type = "input",
				name = L["Separator"],
				order = 3,
				get = "getSeparator",
				set = "setSeparator",
				width = "full",
			},
			healerList = {
				type = "input",
				name = L["Healer List"],
				order = 4,
				get = "getHealerList",
				set = "setHealerList",
				width = "full",
			},
		  },
		},
		prepareEncounter = {
			type = "execute",
			order = 7,
			name = L["Prepare Encounter"],
			func = "PrepareEncounter",
				width = 1.1,
		},
		toggleAnchor = {
			type = "execute",
			order = 8,
			name = L["Toggle Anchor"],
			func = "ToggleAnchor",
				width = 1.1,
		},
	}
}

function JeypLoathebHealSorter:HasAssist() 
	return IsInRaid("player") and (UnitIsGroupAssistant("player") or UnitIsGroupLeader("player"))
end

function JeypLoathebHealSorter:CanSendData() 
	return (self:HasAssist() and self.db.global.bSendData)
end

function JeypLoathebHealSorter:AddToLogFn(_sFunction, _sParam1, _sParam2, _sParam3) 
	self:AddToLogBase("FunctionCall", {
		sFunction = _sFunction,
		sParam1 = _sParam1,
		sParam2 = _sParam2,
		sParam3 = _sParam3,
	})
end

function JeypLoathebHealSorter:AddToLogBase(_sLogType, tPartialLog) 
	if tPartialLog == null then tPartialLog = {} end
	
	tPartialLog.sDateTime = date("%Y %B %d - %H:%M:%S")
	tPartialLog.sLogType = _sLogType
	
	local sDate = date("%Y%m%d")
	
	if self.db.global.tLog[sDate] == nil then 
		self.db.global.tLog[sDate] = {} 
	end
	table.insert(self.db.global.tLog[sDate], tPartialLog)
end

function JeypLoathebHealSorter:AddToLogInit(_sName, _sValue) 
	self:AddToLogBase("Init", {
		sName = _sName,
		sValue = _sValue,
	})
end

function JeypLoathebHealSorter:AddToLogEditValue(_sName, _sOldValue, _sNewValue) 
	self:AddToLogBase("EditValue", {
		sName = _sName,
		sOldValue = _sOldValue,
		sNewValue = _sNewValue,
	})
end

function JeypLoathebHealSorter:AddToLogComm(_sSender, _sEvent, _fTimestamp, _sMessage) 
	local sAddonVersionID = "Unknown"
	if self.tHealerList[_sSender] ~= nil then 
		sAddonVersionID = self.tHealerList[_sSender].sAddonVersionID
	end
	
	self:AddToLogBase("OnCommReceived", {
		sMessageEvent = _sEvent,
		sAddonVersionID = sAddonVersionID,
		fMessageTimestamp = _fTimestamp,
		sMessageSender = _sSender,
		sMessage = _sMessage
	})
end

function JeypLoathebHealSorter:AddToLogCLEU(_sEvent, _fTimestamp, _tCLEUParams) 
	self:AddToLogBase("CLEU", {
		sEvent = _sEvent,
		fEventTimestamp = _fTimestamp,
		sSourceName = _tCLEUParams.sourceName,
		sSourceGUID = _tCLEUParams.sourceGUID,
		sDestName = _tCLEUParams.destName,
		sDestGUID = _tCLEUParams.destGUID,
		sSpellName = _tCLEUParams.spellName
	})
end

function JeypLoathebHealSorter:ClearOldLogs() 
	local day, month, year = (date("%d-%m-%Y")):match("(%d%d)-(%d%d)-(%d%d%d%d)")
	local endtime = time({day = day - 1, month = month, year = year})
	local iStartDateLog = tonumber(date("%Y%m%d", endtime))
	
	for logDayDate, logDayData in pairs(self.db.global.tLog) do
		if tonumber(logDayDate) < iStartDateLog then 
			self.db.global.tLog[logDayDate] = nil
		end
	end
end

function JeypLoathebHealSorter:ExportDebug() 
	local frame = AceGUI:Create("Frame")
	frame:SetTitle("JeypLoathebHealSorter - Export Window")
	local editbox = AceGUI:Create("MultiLineEditBox")
	editbox:SetLabel("Debug Infos:")
	local s = {"{\n"}
	for logDayDate, logDayData in pairs(self.db.global.tLog) do
		s[#s+1] = "  [" .. logDayDate .. "] = {\n"
		for i=1,#logDayData do
		  s[#s+1] = "    {\n"
		  for k,v in pairs(logDayData[i]) do
			s[#s+1] = "      ["
			s[#s+1] = k
			s[#s+1] = "]:["
			s[#s+1] = tostring(v)
			s[#s+1] = "],\n"
		  end
		  s[#s+1] = "    },\n"
		end
		s[#s+1] = "  },\n"
	end
	s[#s+1] = "}"
	s = table.concat(s)
	
	editbox:SetText(s)
	editbox:SetFullWidth(true)
	editbox:SetFullHeight(true)
	editbox:SetNumLines(25)
	frame:AddChild(editbox)
end

function JeypLoathebHealSorter:setMargin(info, value)
	self.db.global.iMargin = value
	self:ReloadFrame(self.db.global.iHealersToDisplay, self.db.global.iHealersToDisplay)
end

function JeypLoathebHealSorter:getMargin(info)
	return self.db.global.iMargin;
end

function JeypLoathebHealSorter:setBorderColor(info, r, g, b, a)
	self.db.global.tBorderColor.r = r;
	self.db.global.tBorderColor.g = g;
	self.db.global.tBorderColor.b = b;
	self.db.global.tBorderColor.a = a;
	self:ReloadBackdrop()
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getBorderColor(info)
	return self.db.global.tBorderColor.r, self.db.global.tBorderColor.g, self.db.global.tBorderColor.b, self.db.global.tBorderColor.a;
end

function JeypLoathebHealSorter:setSeparator(info, value)
	self:AddToLogEditValue("sSeparator", self.db.global.sSeparator, value) 
	self.db.global.sSeparator = string.sub(value, 1, 1);
	self.db.global.iDataTimestamp = GetServerTime();
	if self:CanSendData() then
		if self.iGreatestTimestamp == self.db.global.iDataTimestamp then
			self.fVerifyHealerListTime = GetTime() + 1 -- Delay verification to ensure it only runs once
		end
		
		self:SendHealerList()
	else
		self:ReLoadHealerList()
	end
end

function JeypLoathebHealSorter:getSeparator(info)
	return self.db.global.sSeparator;
end

function JeypLoathebHealSorter:setBorder(info, value)
	self.db.global.sBorder = value;
	self:ReloadBackdrop()
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getBorder(info)
	return self.db.global.sBorder;
end


function JeypLoathebHealSorter:setBackgroundColor(info, r, g, b, a)
	self.db.global.tBackgroundColor.r = r;
	self.db.global.tBackgroundColor.g = g;
	self.db.global.tBackgroundColor.b = b;
	self.db.global.tBackgroundColor.a = a;
	self:ReloadBackdrop()
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getBackgroundColor(info)
	return self.db.global.tBackgroundColor.r, self.db.global.tBackgroundColor.g, self.db.global.tBackgroundColor.b, self.db.global.tBackgroundColor.a;
end

function JeypLoathebHealSorter:setBackground(info, value)
	self.db.global.sBackground = value;
	self:ReloadBackdrop()
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getBackground(info)
	return self.db.global.sBackground;
end

function JeypLoathebHealSorter:setFont(info, value)
	self.db.global.sFont = value;
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getFont(info)
	return self.db.global.sFont;
end

function JeypLoathebHealSorter:setFontSize(info, value)
	self.db.global.iFontSize = value;
	self.fMargin = value * 0.5
	self:UpdateFontSize()
end

function JeypLoathebHealSorter:getFontSize(info)
	return self.db.global.iFontSize;
end

function JeypLoathebHealSorter:setHealerList(info, value)
	self:AddToLogEditValue("sHealerList", self.db.global.sHealerList, value) 
	self.db.global.sHealerList = value;
	self.db.global.iDataTimestamp = GetServerTime();
	if self:CanSendData() then
		if self.iGreatestTimestamp == self.db.global.iDataTimestamp then
			self.fVerifyHealerListTime = GetTime() + 1 -- Delay verification to ensure it only runs once
		end
		
		self:SendHealerList()
	end
end

function JeypLoathebHealSorter:getHealerList(info)
	return self.db.global.sHealerList;
end

function JeypLoathebHealSorter:setSendData(info, value)
	self:AddToLogEditValue("bSendData", self.db.global.bSendData, value) 
	self.db.global.bSendData = value;
	if self:CanSendData() then
		self:SendHealerList()
	end
end

function JeypLoathebHealSorter:getSendData(info)
	return self.db.global.bSendData;
end

function JeypLoathebHealSorter:setRaidWarningCallouts(info, value)
	self:AddToLogEditValue("bRaidWarningCallouts", self.db.global.iRaidWarningCallouts, value) 
	self.db.global.iRaidWarningCallouts = value;
end

function JeypLoathebHealSorter:getRaidWarningCallouts(info)
	return self.db.global.iRaidWarningCallouts;
end

function JeypLoathebHealSorter:setIgnoreNonParty(info, value)
	self.db.global.bIgnoreNonParty = value;
	self:ReLoadHealerList()
end

function JeypLoathebHealSorter:getIgnoreNonParty(info)
	return self.db.global.bIgnoreNonParty;
end

function JeypLoathebHealSorter:setTestMode(info, value)
	self:AddToLogEditValue("bTestMode", self.db.global.bTestMode, value) 
	self.db.global.bTestMode = value;
	self:SetSpellNames()
	self:DisableWindow(true)
	self:CheckIfActiveAlreadyPresent()
end

function JeypLoathebHealSorter:getTestMode(info)
	return self.db.global.bTestMode;
end

function JeypLoathebHealSorter:setHealersToDisplay(info, value)
	local iOldValue = self.db.global.iHealersToDisplay
	self.db.global.iHealersToDisplay = value;
	self:ReloadFrame(iOldValue, value)
end

function JeypLoathebHealSorter:getHealersToDisplay(info)
	return self.db.global.iHealersToDisplay;
end

function JeypLoathebHealSorter:resetFramePosition(info)
	self.Frame:ClearAllPoints()
	self.Frame:SetPoint("CENTER", 0, 0)
end

function JeypLoathebHealSorter:InitOptions() 
	self.db = LibStub("AceDB-3.0"):New("JeypLoathebHealSorterOptions", defaults, "Default");
	LibStub("AceConfig-3.0"):RegisterOptionsTable("JeypLoathebHealSorter", JeypLoathebHealSorter.options);
	self.JeypLoathebHealSorterOptions = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("JeypLoathebHealSorter", "JeypLoathebHealSorter");
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("JeypLoathebHealSorter", 420, 620)
end